/**
 * 
 */
package com.test;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Properties;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.agent.KnowledgeAgent;
import org.drools.agent.KnowledgeAgentFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.command.Command;
import org.drools.command.CommandFactory;
import org.drools.definition.KnowledgePackage;
import org.drools.definition.rule.Rule;
import org.drools.io.ResourceFactory;
import org.drools.runtime.ExecutionResults;
import org.drools.runtime.StatelessKnowledgeSession;



public class DSLTest {

	private void dumpKBase(KnowledgeBase kbase) {
		System.out.println("Knowledge Base rules:\n=============================");
		Collection<KnowledgePackage> pkgs = kbase.getKnowledgePackages();
		for (KnowledgePackage kp : pkgs) {
			for (Rule rule : kp.getRules()) {
				System.out.println("Rule: " + rule.getName() + ", " + rule.getPackageName());
				for (String attribs : rule.listMetaAttributes())
					System.out.println("---- " + attribs);
			}
		}
		System.out.println("=============================");
	}

	private void runScenario (StatelessKnowledgeSession session) {
		
		// Create data
		Person infant = new Person("Baby Joe", AgeRange.INFANT);
		Person youth = new Person("Young Jane", AgeRange.YOUTH);
		Person adult = new Person("Middle Aged Matt", AgeRange.ADULT);
		Person senior = new Person("Senior Betty", AgeRange.SENIOR);
		
		// Execute Rules
		ArrayList<Command> cmds = new ArrayList<Command>();
		cmds.add(CommandFactory.newInsert(infant));
		cmds.add(CommandFactory.newInsert(youth));
		cmds.add(CommandFactory.newInsert(adult));
		cmds.add(CommandFactory.newInsert(senior));
		cmds.add(CommandFactory.newFireAllRules());

		ExecutionResults results = session.execute(CommandFactory.newBatchExecution(cmds));
		System.out.println("------------------------\nRule Execution Complete\n------------------------");
	}
	
	public void noChangeSetTest() throws Exception { 

		System.out.println("Executing Rules after Manual KnowledgeBase Creation");
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		
		kbuilder.add(ResourceFactory.newClassPathResource("com/dsl/demo.dsl"), ResourceType.DSL);
		kbuilder.add(ResourceFactory.newClassPathResource("com/dsl/test-dsl.dslr"), ResourceType.DSLR);
		kbuilder.add(ResourceFactory.newClassPathResource("com/nondsl/non-dsl.drl"), ResourceType.DRL);

		if (kbuilder.hasErrors()) {
			throw new Exception(kbuilder.getErrors().toString());
		}

		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		dumpKBase(kbase);

		runScenario(kbase.newStatelessKnowledgeSession());		
	}
	
	private Properties getScannerConfig(String resourceLocation) throws IOException {
		Properties changeConfig = new Properties();
		InputStream garbIn = null;
		
		try {
			garbIn = DSLTest.class.getResourceAsStream(resourceLocation);
			changeConfig.load(garbIn);
		}
		finally { 
			if (garbIn != null)
				garbIn.close();
		}
		
		return changeConfig; 
	}
	
	public void changeSetTest() throws Exception {
		
		System.out.println("Executing Rules after KnowledgeAgent based KnowledgeBase Creation");
		String agentName = "dsl-test";
		
		KnowledgeAgent ka = KnowledgeAgentFactory.newKnowledgeAgent(agentName);
		ka.applyChangeSet(ResourceFactory.newClassPathResource(agentName + "-changeset.xml", getClass()));
		
		dumpKBase(ka.getKnowledgeBase());
		
		runScenario(ka.newStatelessKnowledgeSession());
	}
	
	public static void main (String args[]) throws Exception {
		
		DSLTest testDrive = new DSLTest();
		
		testDrive.noChangeSetTest();
		testDrive.changeSetTest();
	}

}
